#ifndef __BRAIN_MULTIPLAYER_MOG2
#define __BRAIN_MULTIPLAYER_MOG2


extern int MAX_HYPOTHESES;


/* Information of one player: */ 
class NWMOG2PlayerInfo {
public:
	int player_ID;			/* Has to be unique among all the players in the game */ 
	OBJECT *avatar;
	char name[80];
	int left_key,right_key,up_key,down_key,fire1_key,fire2_key,fire3_key,fire4_key;

	ROOM *start_room;
	MAP *start_map;
	MOG2_INTERFACE *start_interface;

	SDL_Rect vp_area;
	float vp_zoom;
};


/* Information of one client/server: */ 
class NWMOG2ClientInfo {
public:
	NWMOG2ClientInfo();
	~NWMOG2ClientInfo();

	List<NWMOG2PlayerInfo> players;
	int LVK_ID;	/* Has to be unique among all the computers in the game */ 

	char computer_name[80];	   /* NICK */ 
	char computer_address[80]; /* IP Address */ 	
	TCPsocket socket;

	/* General: */ 
	NWMOG2PlayerInfo *find_by_player_identifier(int ID);
	int get_nplayers(void);
};


/* Gomplete Network game configuration: */ 
class NWMOG2GameInfo {
public:
	NWMOG2GameInfo(MOG2 *g);
	~NWMOG2GameInfo();

	MOG2 *game;

	int port;
	SDLNet_SocketSet socket_set;
	TCPsocket server_socket;

	List<NWMOG2ClientInfo> clients;
	NWMOG2ClientInfo *server;
	NWMOG2ClientInfo *local;

	int render; /* 0 = software, 1 = hardware */ 
	int screen_x,screen_y;

	/* general: */ 
	int new_LVK_identifier(void);
	int new_player_identifier(void);

	int get_local_nplayers(void);
	int get_nplayers(void);

	NWMOG2ClientInfo *find_by_LVK_identifier(int ID);
	NWMOG2PlayerInfo *find_by_player_identifier(int ID);

};



/* Class that manages the networked gameplay: */ 
class NetworkMOG2Protocol {
public:
	NetworkMOG2Protocol(NWMOG2GameInfo *gi,int game_mode,char *file_name);
	virtual ~NetworkMOG2Protocol();

	bool listening_cycle(void);
	bool execute_cycle(void);
	bool event(SDL_Event *event);

	virtual bool game_hypothetic_cycle(class KEYBOARDSTATE **LVK,int n_LVK,int n_cycle);
	virtual bool game_update_hypotheses(KEYBOARDSTATE **LVK,int n_LVK,int n_cycle);

	int game_mode;
	FILE *replay_file;

	// Common:
	NWMOG2GameInfo *gameinfo;
	int current_cycle,last_synchronized_cycle;
	class KEYBOARDSTATE **LVK;	/* Contains the LASTEST information known about the LVKs */ 
	int *LVK_cycle;
	List<KEYBOARDSTATE> *hypothetic_LVK;
	bool bloqued;

	// Client:
	unsigned int network_timeout_start,network_timeout;
	int connection_state;

	// Server:
	bool *client_active;
};


class ActiveNetworkMOG2 : public NetworkMOG2Protocol {
public:
	ActiveNetworkMOG2(NWMOG2GameInfo *gi,int game_mode,char *file_name);

	virtual bool game_hypothetic_cycle(KEYBOARDSTATE **LVK,int n_LVK,int n_cycle);
	virtual bool game_update_hypotheses(KEYBOARDSTATE **LVK,int n_LVK,int n_cycle);
	
	ActiveMOG2 *game;
};




#endif

